"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Lincoln = void 0;
var uuid_1 = require("uuid");
var events_1 = require("events");
var collections_1 = require("@nofrills/collections");
var LogMessageType_1 = require("./LogMessageType");
var LincolnEvents_1 = require("./LincolnEvents");
var DefaultOptions = {
    emitNamespace: true,
    emitTag: true,
    filters: new collections_1.Registry(),
    interceptors: new collections_1.Registry(),
    namespace: 'app',
    separator: ':',
};
var Lincoln = /** @class */ (function (_super) {
    __extends(Lincoln, _super);
    function Lincoln(logOptions, loggers) {
        if (loggers === void 0) { loggers = []; }
        var _this = _super.call(this) || this;
        _this.loggers = loggers;
        _this.id = uuid_1.v4();
        if (logOptions && typeof logOptions === 'string') {
            logOptions = { namespace: logOptions };
        }
        var current = logOptions || DefaultOptions;
        _this.options = __assign(__assign({}, DefaultOptions), current);
        return _this;
    }
    Object.defineProperty(Lincoln.prototype, "namespace", {
        get: function () {
            return this.options.namespace;
        },
        enumerable: false,
        configurable: true
    });
    Lincoln.prototype.debug = function () {
        var parameters = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parameters[_i] = arguments[_i];
        }
        this.write(LogMessageType_1.LogMessageType.debug, parameters);
    };
    Lincoln.prototype.error = function () {
        var parameters = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parameters[_i] = arguments[_i];
        }
        this.write(LogMessageType_1.LogMessageType.error, parameters);
    };
    Lincoln.prototype.fatal = function () {
        var parameters = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parameters[_i] = arguments[_i];
        }
        this.write(LogMessageType_1.LogMessageType.fatal, parameters);
    };
    Lincoln.prototype.info = function () {
        var parameters = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parameters[_i] = arguments[_i];
        }
        this.write(LogMessageType_1.LogMessageType.info, parameters);
    };
    Lincoln.prototype.silly = function () {
        var parameters = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parameters[_i] = arguments[_i];
        }
        this.write(LogMessageType_1.LogMessageType.silly, parameters);
    };
    Lincoln.prototype.trace = function () {
        var parameters = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parameters[_i] = arguments[_i];
        }
        this.write(LogMessageType_1.LogMessageType.trace, parameters);
    };
    Lincoln.prototype.warn = function () {
        var parameters = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parameters[_i] = arguments[_i];
        }
        this.write(LogMessageType_1.LogMessageType.warn, parameters);
    };
    Lincoln.prototype.extend = function (tag) {
        return new Lincoln(__assign(__assign({}, this.options), { namespace: this.normalize(tag) }));
    };
    Lincoln.prototype.now = function () {
        return new Date().toISOString();
    };
    Lincoln.prototype.normalize = function (tag) {
        if (this.options.emitNamespace && this.options.emitTag) {
            return "" + this.namespace + this.options.separator + tag;
        }
        else if (this.options.emitNamespace === false && this.options.emitTag) {
            return tag;
        }
        else {
            return this.namespace;
        }
    };
    Lincoln.prototype.write = function (tag, parameters) {
        var _this = this;
        var original = {
            id: uuid_1.v4(),
            namespace: this.normalize(tag),
            parameters: parameters,
            timestamp: Date.now(),
            type: tag,
        };
        var log = Array.from(this.options.interceptors.values).reduce(function (result, interceptor) { return interceptor(result); }, original);
        var filtered = Array.from(this.options.filters.values)
            .map(function (filter) { return filter(log); })
            .reduce(function (result, current) { return (result ? result : current); }, false);
        if (filtered === true) {
            this.emit(LincolnEvents_1.LincolnEvents.Filtered, log);
            return;
        }
        this.loggers.map(function (logger) { return _this.writer(logger, log); });
        this.emit(LincolnEvents_1.LincolnEvents.Log, log, this.loggers.length);
    };
    Lincoln.prototype.writer = function (logger, log) {
        if (logger.write(log)) {
            this.emit(LincolnEvents_1.LincolnEvents.Written, log);
        }
    };
    return Lincoln;
}(events_1.EventEmitter));
exports.Lincoln = Lincoln;
//# sourceMappingURL=Lincoln.js.map